import { EditorView, basicSetup } from 'codemirror';
import { javascript } from '@codemirror/lang-javascript';
import { json } from '@codemirror/lang-json';

let editor: EditorView = null;

type CreateOpts = {
  jsx?: boolean;
  typescript?: boolean;
  type?: 'javascript' | 'json';
};
/**
 * 创建单例
 * @param el
 * @returns
 */
const createEditorInstance = (el?: HTMLDivElement, opts?: CreateOpts) => {
  if (editor && el) {
    el.appendChild(editor.dom);
    return editor;
  } else if (editor) {
    return editor;
  }
  const { type = 'javascript' } = opts || {};
  const plugins = [basicSetup];
  if (type === 'json') {
    plugins.push(json());
  } else {
    plugins.push(javascript({ jsx: opts?.jsx, typescript: opts?.typescript }));
  }
  editor = new EditorView({
    extensions: plugins,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

/**
 * 每次都创建新的实例
 * @param el
 * @returns
 */
export const createEditor = (el: HTMLDivElement, opts?: CreateOpts) => {
  const { type = 'javascript' } = opts || {};
  const plugins = [basicSetup];
  if (type === 'json') {
    plugins.push(json());
  } else {
    plugins.push(javascript({ jsx: opts?.jsx, typescript: opts?.typescript }));
  }
  const editor = new EditorView({
    extensions: plugins,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

export { editor, createEditorInstance };
